"""Вычислитель выражения."""

import typing as tp

from .btree import BTree, Node
from .parser import (
    CLOSING_BRACKET, OPENING_BRACKET, OPERATORS,
    lexer, Token, SyntaxError,
)


def _compile(token_generator: tp.Generator[Token, None, None]) -> BTree:
    """Компиляция токенов в дерево выражения."""
    tree = BTree()
    current = None
    for token in token_generator:
        node = None
        if token.text == OPENING_BRACKET:
            node = _compile(token_generator).root
            setattr(node, "enclosed", True)
        elif token.text == CLOSING_BRACKET:
            break
        else:
            node = Node(token)
        if tree.root is None:
            tree.root = node
            current = tree.root
        elif current.value.text in OPERATORS.keys():
            if current.right is None:
                if (
                    node.value.text in OPERATORS.keys() and
                    not getattr(node, "enclosed", False)
                ):
                    raise SyntaxError(node.value.pos)
                current.right = node
                node.parent = current
                while current.parent is not None:
                    current = current.parent
            else:
                if (
                    not node.is_leaf or
                    node.value.text not in OPERATORS.keys()
                ):
                    raise SyntaxError(token.pos)
                if (
                    getattr(current, "enclosed", False) or
                    node.value.priority <= current.value.priority
                ):
                    node.left = current
                    node.parent = current.parent
                    current.parent = node
                    current = node
                else:
                    node.left = current.right
                    current.right.parent = node
                    current.right = node
                    node.parent = current
                    current = node
        else:
            if current.left is not None:
                raise SyntaxError(token.pos)
            node.left = current
            node.parent = current.parent
            current.parent = node
            current = node
        if current.parent is None:
            tree.root = current
    return tree


def compile(expression: str) -> BTree:
    """Компилятор выражения."""
    return _compile(lexer(expression))


def _calc(
    token: Token,
    left: tp.Optional[int],
    right: tp.Optional[int],
) -> tp.Optional[int]:
    """Вычислитель элемента выражения."""
    return token.eval(left, right)


def _calculate(tree: BTree) -> tp.Optional[int]:
    """Вычисление скомпилированного дерева выражения."""
    return tree.visit(_calc)


def calculate(expression: str) -> tp.Optional[int]:
    return _calculate(compile(expression))
