"""Бинарное дерево."""

from __future__ import annotations
import typing as tp

from .parser import Token


class Node(object):
    """Узел бинарного дерева."""

    value: Token
    left: tp.Optional[Node]
    right: tp.Optional[Node]
    parent: tp.Optional[Node]

    def __init__(
        self,
        value: Token,
        parent: tp.Optional[Node] = None,
        left: tp.Optional[Node] = None,
        right: tp.Optional[Node] = None,
    ) -> None:
        super().__init__()
        self.value = value
        self.parent = parent
        self.left = left
        self.right = right

    @property
    def is_leaf(self) -> bool:
        """Возвращает истина, если этот узел является листом дерева."""
        return self.left is None and self.right is None

    def __str__(self) -> str:
        if self.is_leaf:
            return str(self.value)
        else:
            return f"{self.value}({self.left} {self.right})"

    def __eq__(self, o: object) -> bool:
        if isinstance(o, Node):
            return (
                self.value == o.value and
                self.left == o.left and
                self.right == o.right
            )
        return False


def _visit(
    node: Node,
    func: tp.Callable[
        [Token, tp.Optional[int], tp.Optional[int]],
        tp.Optional[int],
    ],
) -> tp.Optional[int]:
    """Визитёр узла дерева."""
    left = _visit(node.left, func) if node.left is not None else None
    right = _visit(node.right, func) if node.right is not None else None
    return func(node.value, left, right)


class BTree(object):
    """Двоичное дерево."""

    root: tp.Optional[Node]

    def __init__(self) -> None:
        super().__init__()
        self.root = None

    def visit(
        self,
        func: tp.Callable[
            [Token, tp.Optional[int], tp.Optional[int]],
            tp.Optional[int],
        ],
    ) -> tp.Optional[int]:
        """Обход дерева."""
        return _visit(self.root, func)

    def __str__(self) -> str:
        return str(self.root)

    def __eq__(self, o: object) -> bool:
        if isinstance(o, BTree):
            return self.root == o.root
        return False
