#!/usr/bin/env python
"""
mscalc -- Console calculator.

@author:     Aleksei Badiaev <aleksei.badiaev@mousesoft.ru>
@copyright:  2021 Aleksei Badiaev. All rights reserved.
"""

import os
import sys
import logging
import traceback
import argparse

import mscalc
from .calculator import compile, calculate


__all__ = ("main",)
PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))
DEBUG = False
HELP_FORMATTER_CLASS = argparse.RawDescriptionHelpFormatter
logger = logging.getLogger("mscalc")


def main(argv=None):
    """Точка входа в приложение."""
    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % mscalc.__version__
    program_build_date = str(mscalc.__updated__)
    program_version_message = "%%(prog)s %s (%s)" % (
        program_version,
        program_build_date,
    )
    program_shortdesc = __import__("mscalc").cli.__doc__.split("\n")[1]
    program_license = """%s

  Created by Aleksei Badiaev <aleksei.badyaev@gmail.com> on %s.
  Copyright 2015 Aleksei Badiaev. All rights reserved.

  Distributed on an 'AS IS' basis without warranties
  or conditions of any kind, either express or implied.
""" % (
        program_shortdesc,
        str(mscalc.__date__),
    )

    parser = argparse.ArgumentParser(
        prog=program_name,
        description=program_license,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=program_version_message,
    )
    parser.add_argument(
        "expression",
        nargs="+",
        help="Арифметическое выражение.",
    )
    params = parser.parse_args()

    try:
        print(calculate("".join(params.expression)))
    except Exception as ex:
        logger.error("%s: %s", type(ex).__name__, str(ex))
        traceback.print_exc(file=sys.stderr)
        return 1
    return 0


if __name__ == "main":
    sys.exit(main())
